#!/usr/bin/env python
#
# -------------------------------------------------------------------------------------
#
# Copyright (c) 2021 Skinnyhouse
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification, 
# are permitted provided that the following conditions are met:
#
#    - Redistributions of source code must retain the above copyright notice, this 
#    list of conditions and the following disclaimer.
#    - Redistributions in binary form must reproduce the above copyright notice, 
#    this list of conditions and the following disclaimer in the documentation and/or 
#    other materials provided with the distribution.
#    - Neither the name of the author nor the names of its contributors may be used 
#    to endorse or promote products derived from this software without specific prior 
#    written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY 
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
# SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
# TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH 
# DAMAGE.
#
# -------------------------------------------------------------------------------------
#
# This file is a basic Python plug-in for GIMP.
# Requested by GJL to script his method for creating seamless tiles
# It can be executed by selecting the menu option: Filters>>SeamStress1.1

# Version 1 Two Options for making seamless tiles
#			
# Version 1.1
#			Correctly handles transparent background with Quarter Cut tiles
#			Adds options for traditional pattern making - half brick, half drop, turnover types
#
# Version 1.2
#			Adds option to flip layers for half drop and half brick
#			Applies meaningful layer names
# Version 1.3
#           Adds options to create a second new image filled with the new tiles


from gimpfu import *
#from array import array
import math



def do_stuff(image, layer, pattern_type, angle, rotation, flip, size, do_exemplar, i_width, i_height, t_size):
    #### Main Filter Proc
	
    # Store the GIMP's settings so they can be restored
    gimp.context_push()
    # Set start undo point
    #pdb.gimp_image_undo_group_start(image)	
    
	
    num_layers, layer_ids = pdb.gimp_image_get_layers(image)
    layer_names_list = image.layers
    visi_loop, empty = 0, True
		
    while visi_loop < num_layers:
        visi = pdb.gimp_item_get_visible(layer_names_list[visi_loop])
        if visi: empty = False
        visi_loop += 1
			
    #pdb.gimp_message("empty = " + str(empty) )
    success = pdb.gimp_edit_copy_visible(image)
    if empty: success = False
    pdb.gimp_selection_none(image)
    if success:
        flip-=1
        if pattern_type==0: # Diagonal tile 
            angle = math.radians(angle)
		
            # create new image and add visible contents of current image as new layer
            new_image = pdb.gimp_image_new(640, 640, 0)
			
            # Set start undo point
            pdb.gimp_image_undo_group_start(new_image)	
			
            original_layer = pdb.gimp_layer_new_from_visible(image, new_image, "Pattern")
            new_image.add_layer(original_layer, -1)
			
            floating_sel = pdb.gimp_edit_paste(original_layer, False)
            pdb.gimp_floating_sel_anchor(floating_sel)		
		
		
            new_display = pdb.gimp_display_new(new_image)
		
            # ensure that layer will be square
            if original_layer.height<>original_layer.width:
                adj_bit = 0
                if original_layer.height>original_layer.width:
                    if original_layer.width % 2 == 1: adj_bit = -1
                    pdb.gimp_layer_resize(original_layer, original_layer.width + adj_bit, original_layer.width + adj_bit,0, 0)
                else:
                    if original_layer.height % 2 == 1: adj_bit = -1
                    pdb.gimp_layer_resize(original_layer, original_layer.height + adj_bit, original_layer.height + adj_bit,0, 0)
		
            # scale layer 
            pdb.gimp_layer_scale(original_layer, 940, 940, False)
		
            # and centre within image
            pdb.gimp_layer_set_offsets(original_layer, (new_image.width - original_layer.width)/2, (new_image.height - original_layer.height)/2)
		
            # rotate layer and crop to content
            pdb.gimp_item_transform_rotate(original_layer, angle, True, 0, 0)
            pdb.plug_in_autocrop_layer(new_image, original_layer)
		
            pdb.gimp_layer_resize_to_image_size(original_layer)
		
            # scale layer 
            pdb.gimp_layer_scale(original_layer, new_image.width/2, new_image.height/2, False)
		
            # copy layer and add to image
            first_copy = pdb.gimp_layer_new_from_drawable(original_layer, new_image)
            new_image.add_layer(first_copy, -1)
		
            # flip horizontal
            pdb.gimp_item_transform_flip_simple(first_copy, 0, True, 0)
            # and position
            pdb.gimp_layer_set_offsets(first_copy, (new_image.width)/2, 0)
		
            # copy next layer and add to image
            second_copy = pdb.gimp_layer_new_from_drawable(first_copy, new_image)
            new_image.add_layer(second_copy, -1)
		
            # flip vertical
            pdb.gimp_item_transform_flip_simple(second_copy, 1, True, 0)
            # and position
            pdb.gimp_layer_set_offsets(second_copy, (new_image.width)/2, (new_image.height)/2)
		
            # copy next layer and add to image
            third_copy = pdb.gimp_layer_new_from_drawable(second_copy, new_image)
            new_image.add_layer(third_copy, -1)
		
            # flip horizontal
            pdb.gimp_item_transform_flip_simple(third_copy, 0, True, 0)
            # and position
            pdb.gimp_layer_set_offsets(third_copy, 0, (new_image.height)/2)
		
		
            merged_layer = pdb.gimp_image_merge_visible_layers(new_image,1)
		
		
            #scale pattern to user size
            pdb.gimp_image_scale(new_image, size, size)
		
		
            pdb.gimp_item_set_name(merged_layer, "Diagonal tile")
		
            # Set end undo point
            pdb.gimp_image_undo_group_end(new_image)
		
        elif pattern_type==1: # Quarter cut and swap
        
            new_image = pdb.gimp_edit_paste_as_new_image()
            new_display = pdb.gimp_display_new(new_image)
			
            # Set start undo point
            pdb.gimp_image_undo_group_start(new_image)	
			
            original_layer = new_image.active_layer
            pdb.gimp_item_set_name(original_layer, "Cut and swap block")
            # ensure that image will be square
            if new_image.height<>new_image.width:
                adj_bit = 0
                if new_image.height>new_image.width:
                    if new_image.width % 2 == 1: adj_bit = -1
                    pdb.gimp_image_resize(new_image, new_image.width + adj_bit, new_image.width + adj_bit,0, 0)
                else:
                    if new_image.height % 2 == 1: adj_bit = -1
                    pdb.gimp_image_resize(new_image, new_image.height + adj_bit, new_image.height + adj_bit,0, 0)
			
            # create new layer from l/h half of the new image
            selection = pdb.gimp_image_select_rectangle(new_image, 2, 0, 0, new_image.width/2, new_image.height)
            success2 = pdb.gimp_edit_copy(original_layer)
            temp_layer = gimp.Layer(new_image, "Temp Layer", new_image.width/2, new_image.height, RGBA_IMAGE, 100, 0) # layer 
            new_image.add_layer(temp_layer, -1)
            floating_sel = pdb.gimp_edit_paste(temp_layer, False)
            pdb.gimp_floating_sel_anchor(floating_sel)		
		
            # create new layer from r/h half of the new image
            selection2 = pdb.gimp_image_select_rectangle(new_image, 2, new_image.width/2, 0, new_image.width, new_image.height)
            temp_layer2 = gimp.Layer(new_image, "Temp Layer2", new_image.width/2, new_image.height, RGBA_IMAGE, 100, 0) # layer 
            new_image.add_layer(temp_layer2, -1)
            success2 = pdb.gimp_edit_copy(original_layer)
            floating_sel = pdb.gimp_edit_paste(temp_layer2, False)
            pdb.gimp_item_transform_translate(floating_sel, new_image.width/4,0)
            pdb.gimp_floating_sel_anchor(floating_sel)
            pdb.gimp_item_transform_translate(temp_layer2, 0-(new_image.width/2),0)
            pdb.gimp_layer_resize(temp_layer2, new_image.width/2, new_image.height, 0-new_image.width/2, 0)
		
            #reposition the two new layers - swap places
            pdb.gimp_layer_set_offsets(temp_layer, new_image.width/2,0)
            pdb.gimp_layer_set_offsets(temp_layer2, 0,0)
		
            # merge the repositioned layers
            temp_layer = pdb.gimp_image_merge_down(new_image, temp_layer2, 1)
                
				
            # create new layer from upper half of the new image
            selection = pdb.gimp_image_select_rectangle(new_image, 2, 0, 0, new_image.width, new_image.height/2)
            success2 = pdb.gimp_edit_copy(temp_layer)
            temp_layer2 = gimp.Layer(new_image, "Temp Layer2", new_image.width, new_image.height/2, RGBA_IMAGE, 100, 0) # layer 
            new_image.add_layer(temp_layer2, -1)
            floating_sel = pdb.gimp_edit_paste(temp_layer2, False)
            pdb.gimp_floating_sel_anchor(floating_sel)
		
                
            # create new layer from lower half of the new image
            selection2 = pdb.gimp_image_select_rectangle(new_image, 2, 0, new_image.height/2, new_image.width, new_image.height)
            success2 = pdb.gimp_edit_copy(temp_layer)
            temp_layer3 = gimp.Layer(new_image, "Temp Layer3", new_image.width, new_image.height, RGBA_IMAGE, 100, 0) # layer 
            new_image.add_layer(temp_layer3, -1)
            floating_sel = pdb.gimp_edit_paste(temp_layer3, False)
            pdb.gimp_floating_sel_anchor(floating_sel)
			
            # resize layer to half height
            pdb.gimp_layer_resize(temp_layer3, (new_image.width), new_image.height/2, 0, 0-(new_image.height/2))		
		
            #reposition the two new layers - swap places
            pdb.gimp_layer_set_offsets(temp_layer2, 0,new_image.height/2)
            pdb.gimp_layer_set_offsets(temp_layer3, 0,0)
		
            # merge the repositioned layers
            temp_layer2 = pdb.gimp_image_merge_down(new_image, temp_layer3, 1)
		
            #scale pattern to user size
            pdb.gimp_image_scale(new_image, size, size)
		
            pdb.gimp_drawable_edit_clear(original_layer)
            pdb.gimp_drawable_edit_clear(temp_layer)
            pdb.gimp_image_merge_visible_layers(new_image,1)
			
			
	
            # Set end undo point
            pdb.gimp_image_undo_group_end(new_image)
			
			
        elif pattern_type==2: # Half brick tile
		
            new_image = pdb.gimp_edit_paste_as_new_image()
            new_display = pdb.gimp_display_new(new_image)
			
            # Set start undo point
            pdb.gimp_image_undo_group_start(new_image)	
			
            original_layer = new_image.active_layer
            pdb.gimp_item_set_name(original_layer, "Half brick block")
            # ensure that image will be square
            if new_image.height<>new_image.width:
                adj_bit = 0
                if new_image.height>new_image.width:
                    if new_image.width % 2 == 1: adj_bit = -1
                    pdb.gimp_image_resize(new_image, new_image.width + adj_bit, new_image.width + adj_bit,0, 0)
                else:
                    if new_image.height % 2 == 1: adj_bit = -1
                    pdb.gimp_image_resize(new_image, new_image.height + adj_bit, new_image.height + adj_bit,0, 0)
					
					
            # create new layer from bottom half of the new image
            selection2 = pdb.gimp_image_select_rectangle(new_image, 2, 0, new_image.height/2, new_image.width, new_image.height)
            temp_layer2 = gimp.Layer(new_image, "Temp Layer2", new_image.width, new_image.height/2, RGBA_IMAGE, 100, 0) # layer 
            new_image.add_layer(temp_layer2, -1)
            success2 = pdb.gimp_edit_copy(original_layer)
            floating_sel = pdb.gimp_edit_paste(temp_layer2, False)
            if flip>-1:
                floating_sel = pdb.gimp_item_transform_flip_simple(floating_sel, flip, True, 0) # layer flip
            pdb.gimp_item_transform_translate(floating_sel, 0,new_image.height/4)
            pdb.gimp_floating_sel_anchor(floating_sel)
            pdb.gimp_item_transform_translate(temp_layer2, 0-(new_image.width/4),0)
            # and resize layer for bottom l/h quarter
            pdb.gimp_layer_resize(temp_layer2, (new_image.width/4)*3, new_image.height/2, 0-(new_image.width/4), 0-(new_image.height/2))
			
			
            # create new layer from bottom half of the new image
            selection3 = pdb.gimp_image_select_rectangle(new_image, 2, 0, new_image.height/2, new_image.width, new_image.height)
            temp_layer3 = gimp.Layer(new_image, "Temp Layer3", new_image.width, new_image.height/2, RGBA_IMAGE, 100, 0) # layer 
            new_image.add_layer(temp_layer3, -1)
            success2 = pdb.gimp_edit_copy(original_layer)
            floating_sel = pdb.gimp_edit_paste(temp_layer3, False)
            if flip>-1:
                floating_sel = pdb.gimp_item_transform_flip_simple(floating_sel, flip, True, 0) # layer flip
            pdb.gimp_item_transform_translate(floating_sel, 0,new_image.height/4)
            pdb.gimp_floating_sel_anchor(floating_sel)
            # and resize layer for bottom r/h quarter
            pdb.gimp_layer_resize(temp_layer3, new_image.width/4, new_image.height/2, 0,0-(new_image.height/2))
            pdb.gimp_item_transform_translate(temp_layer3, ((new_image.width/4))*3,0)
					
            selection4 = pdb.gimp_image_select_rectangle(new_image, 2, 0, (new_image.height/2), (new_image.width), (new_image.height/2))
            pdb.gimp_drawable_edit_clear(original_layer)
            pdb.gimp_selection_none(new_image)
			
            # and merge visible layers
            pdb.gimp_image_merge_visible_layers(new_image,1)
			
            #scale pattern to user size
            pdb.gimp_image_scale(new_image, size, size)
			
            pdb.gimp_image_resize_to_layers(new_image)
		
	
            # Set end undo point
            pdb.gimp_image_undo_group_end(new_image)
			
			
        elif pattern_type==3: # Half drop
		
            new_image = pdb.gimp_edit_paste_as_new_image()
            new_display = pdb.gimp_display_new(new_image)
			
            # Set start undo point
            pdb.gimp_image_undo_group_start(new_image)	
			
            original_layer = new_image.active_layer
            pdb.gimp_item_set_name(original_layer, "Half drop block")
            # ensure that image will be square
            if new_image.height<>new_image.width:
                adj_bit = 0
                if new_image.height>new_image.width:
                    if new_image.width % 2 == 1: adj_bit = -1
                    pdb.gimp_image_resize(new_image, new_image.width + adj_bit, new_image.width + adj_bit,0, 0)
                else:
                    if new_image.height % 2 == 1: adj_bit = -1
                    pdb.gimp_image_resize(new_image, new_image.height + adj_bit, new_image.height + adj_bit,0, 0)
					

			
            # create new layer from r/h half of the new image
            selection2 = pdb.gimp_image_select_rectangle(new_image, 2, new_image.width/2, 0, new_image.width, new_image.height)
            temp_layer2 = gimp.Layer(new_image, "Temp Layer2", new_image.width, new_image.height/2, RGBA_IMAGE, 100, 0) # layer 
            new_image.add_layer(temp_layer2, -1)
            success2 = pdb.gimp_edit_copy(original_layer)
            floating_sel = pdb.gimp_edit_paste(temp_layer2, False)
            if flip>-1:
                floating_sel = pdb.gimp_item_transform_flip_simple(floating_sel, flip, True, 0) # layer flip
            pdb.gimp_floating_sel_anchor(floating_sel)
			
            pdb.gimp_layer_resize(temp_layer2, new_image.width/2, (new_image.width/4)*3, 0-(new_image.width/2), 0-(new_image.height/4))
			
			
            selection3 = pdb.gimp_image_select_rectangle(new_image, 2, new_image.width/2, 0, new_image.width, new_image.height)
            success3 = pdb.gimp_edit_copy(original_layer)
            temp_layer3 = gimp.Layer(new_image, "Temp Layer3", new_image.width, new_image.height/2, RGBA_IMAGE, 100, 0) # layer 
            new_image.add_layer(temp_layer3, -1)
            floating_sel = pdb.gimp_edit_paste(temp_layer3, False)
			
            if flip>-1:
                floating_sel = pdb.gimp_item_transform_flip_simple(floating_sel, flip, True, 0) # layer flip
            pdb.gimp_item_transform_translate(floating_sel, 0,new_image.height/4)
            pdb.gimp_floating_sel_anchor(floating_sel)
            pdb.gimp_layer_resize(temp_layer3, new_image.width/2, new_image.height/4,0- (new_image.width)/2,0)
            pdb.gimp_item_transform_translate(temp_layer3,0, (new_image.height/4)*3)
			
			
            selection4 = pdb.gimp_image_select_rectangle(new_image, 2, (new_image.width)/2, 0, (new_image.width)/2, new_image.height)
            pdb.gimp_drawable_edit_clear(original_layer)
            pdb.gimp_selection_none(new_image)
			
            # and merge visible layers
            pdb.gimp_image_merge_visible_layers(new_image,1)
	
            #scale pattern to user size
            pdb.gimp_image_scale(new_image, size, size)
			
            pdb.gimp_image_resize_to_layers(new_image)
		
		
					
            # Set end undo point
            pdb.gimp_image_undo_group_end(new_image)
			
			
        elif pattern_type==4: # Turnover all
			
	
            new_image = pdb.gimp_edit_paste_as_new_image()
            new_display = pdb.gimp_display_new(new_image)
			
            # Set start undo point
            pdb.gimp_image_undo_group_start(new_image)	
			
            original_layer = new_image.active_layer
            pdb.gimp_item_set_name(original_layer, "Turnover all block")
            # ensure that image will be square
            if new_image.height<>new_image.width:
                adj_bit = 0
                if new_image.height>new_image.width:
                    if new_image.width % 2 == 1: adj_bit = -1
                    pdb.gimp_image_resize(new_image, new_image.width + adj_bit, new_image.width + adj_bit,0, 0)
                else:
                    if new_image.height % 2 == 1: adj_bit = -1
                    pdb.gimp_image_resize(new_image, new_image.height + adj_bit, new_image.height + adj_bit,0, 0)
					
					
            rotation += 3			
					
            # create new layer from upper half of the new image for top l/h quarter
            selection0 = pdb.gimp_image_select_rectangle(new_image, 2, 0, 0, new_image.width, new_image.height/2)
            success0 = pdb.gimp_edit_copy(original_layer)
            temp_layer0 = gimp.Layer(new_image, "Temp Layer0", new_image.width, new_image.height/2, RGBA_IMAGE, 100, 0) # layer 
            new_image.add_layer(temp_layer0, -1)
            floating_sel = pdb.gimp_edit_paste(temp_layer0, False)
            pdb.gimp_floating_sel_anchor(floating_sel)
            # and resize layer for top l/h quarter
            pdb.gimp_layer_resize(temp_layer0, new_image.width/2, new_image.height/2,0,0)	
            if not rotation % 4 == 3:
                pdb.gimp_item_transform_rotate_simple(temp_layer0, (rotation % 4), True, 0, 0)
            rotation += 1	
					
					
            # create new layer from upper half of the new image for top r/h quarter
            selection1 = pdb.gimp_image_select_rectangle(new_image, 2, 0, 0, new_image.width, new_image.height/2)
            success1 = pdb.gimp_edit_copy(original_layer)
            temp_layer1 = gimp.Layer(new_image, "Temp Layer1", new_image.width, new_image.height/2, RGBA_IMAGE, 100, 0) # layer 
            new_image.add_layer(temp_layer1, -1)
            floating_sel = pdb.gimp_edit_paste(temp_layer1, False)
            pdb.gimp_floating_sel_anchor(floating_sel)
            pdb.gimp_item_transform_translate(temp_layer1,(0- new_image.width)/2, new_image.height/2)
            # and resize layer for bottom r/h quarter
            pdb.gimp_layer_resize(temp_layer1, new_image.width/2, new_image.height/2,0- (new_image.width)/2,0)	
            pdb.gimp_item_transform_translate(temp_layer1,(new_image.width)/2,0- (new_image.height)/2)
            if not rotation % 4 == 3:
                pdb.gimp_item_transform_rotate_simple(temp_layer1, (rotation % 4), True, 0, 0)
            rotation += 1	
					
            # create new layer from bottom half of the new image for bottom r/h quarter
            selection2 = pdb.gimp_image_select_rectangle(new_image, 2, 0, new_image.height/2, new_image.width, new_image.height)
            temp_layer2 = gimp.Layer(new_image, "Temp Layer2", new_image.width, new_image.height/2, RGBA_IMAGE, 100, 0) # layer 
            new_image.add_layer(temp_layer2, -1)
            success2 = pdb.gimp_edit_copy(original_layer)
            floating_sel = pdb.gimp_edit_paste(temp_layer2, False)
            pdb.gimp_item_transform_translate(floating_sel, 0,new_image.height/4)
            pdb.gimp_floating_sel_anchor(floating_sel)
            pdb.gimp_item_transform_translate(temp_layer2, 0-(new_image.width/2),0)
            # and resize layer for bottom r/h quarter
            pdb.gimp_layer_resize(temp_layer2, new_image.width/2, new_image.height/2, 0-(new_image.width/2),0-(new_image.height/2))	
            pdb.gimp_item_transform_translate(temp_layer2, new_image.width/2,0)
            if not rotation % 4 == 3:
                pdb.gimp_item_transform_rotate_simple(temp_layer2, (rotation % 4), True, 0, 0)
            rotation += 1	
			
			
            # create new layer from bottom half of the new image for bottom l/h quarter
            selection3 = pdb.gimp_image_select_rectangle(new_image, 2, 0, new_image.height/2, new_image.width, new_image.height)
            temp_layer3 = gimp.Layer(new_image, "Temp Layer3", new_image.width, new_image.height/2, RGBA_IMAGE, 100, 0) # layer 
            new_image.add_layer(temp_layer3, -1)
            success3 = pdb.gimp_edit_copy(original_layer)
            floating_sel = pdb.gimp_edit_paste(temp_layer3, False)
            pdb.gimp_item_transform_translate(floating_sel, 0,new_image.height/4)
            pdb.gimp_floating_sel_anchor(floating_sel)
            # and resize layer for bottom r/h quarter
            pdb.gimp_layer_resize(temp_layer3, new_image.width/2, new_image.height/2, 0,0-(new_image.height/2))	
            if not rotation % 4 == 3:
                pdb.gimp_item_transform_rotate_simple(temp_layer3, (rotation % 4), True, 0, 0)
            rotation += 1	
			
            selection4 = pdb.gimp_image_select_rectangle(new_image, 2, 0, 0, (new_image.width), new_image.height)
            pdb.gimp_drawable_edit_clear(original_layer)
            pdb.gimp_selection_none(new_image)
			
            # and merge visible layers
            pdb.gimp_image_merge_visible_layers(new_image,1)
	
            #scale pattern to user size
            pdb.gimp_image_scale(new_image, size, size)
			
            pdb.gimp_image_resize_to_layers(new_image)
		
		
					
            # Set end undo point
            pdb.gimp_image_undo_group_end(new_image)
			
			
        elif pattern_type==5: # Turnover L>R
			
	
            rotation += 3	
            new_image = pdb.gimp_edit_paste_as_new_image()
            new_display = pdb.gimp_display_new(new_image)
			
            # Set start undo point
            pdb.gimp_image_undo_group_start(new_image)	
			
            original_layer = new_image.active_layer
            pdb.gimp_item_set_name(original_layer, "Turnover L>R block")
            # ensure that image will be square
            if new_image.height<>new_image.width:
                adj_bit = 0
                if new_image.height>new_image.width:
                    if new_image.width % 2 == 1: adj_bit = -1
                    pdb.gimp_image_resize(new_image, new_image.width + adj_bit, new_image.width + adj_bit,0, 0)
                else:
                    if new_image.height % 2 == 1: adj_bit = -1
                    pdb.gimp_image_resize(new_image, new_image.height + adj_bit, new_image.height + adj_bit,0, 0)
					
					
            # create new layer from upper half of the new image for top l/h quarter
            selection1 = pdb.gimp_image_select_rectangle(new_image, 2, 0, 0, new_image.width, new_image.height/2)
            success1 = pdb.gimp_edit_copy(original_layer)
            temp_layer1 = gimp.Layer(new_image, "Temp Layer1", new_image.width, new_image.height/2, RGBA_IMAGE, 100, 0) # layer 
            new_image.add_layer(temp_layer1, -1)
            floating_sel = pdb.gimp_edit_paste(temp_layer1, False)
            pdb.gimp_floating_sel_anchor(floating_sel)
            # and resize layer for top l/h quarter
            pdb.gimp_layer_resize(temp_layer1, new_image.width/2, new_image.height/2,0,0)	
            if not rotation % 4 == 3:
                pdb.gimp_item_transform_rotate_simple(temp_layer1, (rotation % 4), True, 0, 0)
            selection1 = pdb.gimp_image_select_rectangle(new_image, 2, 0, 0, (new_image.width)/2, new_image.height/2)
            pdb.gimp_drawable_edit_clear(original_layer)
            pdb.gimp_selection_none(new_image)
					
            # create new layer from bottom half of the new image for bottom r/h quarter
            selection2 = pdb.gimp_image_select_rectangle(new_image, 2, 0, new_image.height/2, new_image.width, new_image.height)
            temp_layer2 = gimp.Layer(new_image, "Temp Layer2", new_image.width, new_image.height/2, RGBA_IMAGE, 100, 0) # layer 
            new_image.add_layer(temp_layer2, -1)
            success2 = pdb.gimp_edit_copy(original_layer)
            floating_sel = pdb.gimp_edit_paste(temp_layer2, False)
            pdb.gimp_item_transform_translate(floating_sel, 0,new_image.height/4)
            pdb.gimp_floating_sel_anchor(floating_sel)
            pdb.gimp_item_transform_translate(temp_layer2, 0-(new_image.width/2),0)
            # and resize layer for bottom r/h quarter
            pdb.gimp_layer_resize(temp_layer2, new_image.width/2, new_image.height/2, 0-(new_image.width/2),0-(new_image.height/2))	
            pdb.gimp_item_transform_translate(temp_layer2, new_image.width/2,0)
            if not rotation % 4 == 3:
                pdb.gimp_item_transform_rotate_simple(temp_layer2, (rotation % 4), True, 0, 0)
            selection2 = pdb.gimp_image_select_rectangle(new_image, 2, new_image.width/2, (new_image.height/2), (new_image.width)/2, new_image.height/2)
            pdb.gimp_drawable_edit_clear(original_layer)
            pdb.gimp_selection_none(new_image)
					
			
            # and merge visible layers
            pdb.gimp_image_merge_visible_layers(new_image,1)
	
            #scale pattern to user size
            pdb.gimp_image_scale(new_image, size, size)
			
            pdb.gimp_image_resize_to_layers(new_image)
		
		
					
            # Set end undo point
            pdb.gimp_image_undo_group_end(new_image)
			
        else: # Turnover R>L
		
		
            rotation += 3	
            new_image = pdb.gimp_edit_paste_as_new_image()
            new_display = pdb.gimp_display_new(new_image)
			
            # Set start undo point
            pdb.gimp_image_undo_group_start(new_image)	
			
            original_layer = new_image.active_layer
            pdb.gimp_item_set_name(original_layer, "Turnover R>L block")
            # ensure that image will be square
            if new_image.height<>new_image.width:
                adj_bit = 0
                if new_image.height>new_image.width:
                    if new_image.width % 2 == 1: adj_bit = -1
                    pdb.gimp_image_resize(new_image, new_image.width + adj_bit, new_image.width + adj_bit,0, 0)
                else:
                    if new_image.height % 2 == 1: adj_bit = -1
                    pdb.gimp_image_resize(new_image, new_image.height + adj_bit, new_image.height + adj_bit,0, 0)
					
					
            # create new layer from upper half of the new image for top r/h quarter
            selection1 = pdb.gimp_image_select_rectangle(new_image, 2, 0, 0, new_image.width, new_image.height/2)
            success1 = pdb.gimp_edit_copy(original_layer)
            temp_layer1 = gimp.Layer(new_image, "Temp Layer1", new_image.width, new_image.height/2, RGBA_IMAGE, 100, 0) # layer 
            new_image.add_layer(temp_layer1, -1)
            floating_sel = pdb.gimp_edit_paste(temp_layer1, False)
            pdb.gimp_floating_sel_anchor(floating_sel)
            pdb.gimp_item_transform_translate(temp_layer1,(0- new_image.width)/2, new_image.height/2)
            # and resize layer for top r/h quarter
            pdb.gimp_layer_resize(temp_layer1, new_image.width/2, new_image.height/2,0- (new_image.width)/2,0)
            pdb.gimp_item_transform_translate(temp_layer1,(new_image.width)/2,0- (new_image.height)/2)
            if not rotation % 4 == 3:
                pdb.gimp_item_transform_rotate_simple(temp_layer1, (rotation % 4), True, 0, 0)
            selection1 = pdb.gimp_image_select_rectangle(new_image, 2, (new_image.width)/2, 0, (new_image.width)/2, new_image.height/2)
            pdb.gimp_drawable_edit_clear(original_layer)
            pdb.gimp_selection_none(new_image)
				
					
					
            # create new layer from bottom half of the new image for bottom l/h quarter
            selection3 = pdb.gimp_image_select_rectangle(new_image, 2, 0, new_image.height/2, new_image.width, new_image.height)
            temp_layer3 = gimp.Layer(new_image, "Temp Layer3", new_image.width, new_image.height/2, RGBA_IMAGE, 100, 0) # layer 
            new_image.add_layer(temp_layer3, -1)
            success3 = pdb.gimp_edit_copy(original_layer)
            floating_sel = pdb.gimp_edit_paste(temp_layer3, False)
            pdb.gimp_item_transform_translate(floating_sel, 0,new_image.height/4)
            pdb.gimp_floating_sel_anchor(floating_sel)
            # and resize layer for bottom r/h quarter
            pdb.gimp_layer_resize(temp_layer3, new_image.width/2, new_image.height/2, 0,0-(new_image.height/2))	
            if not rotation % 4 == 3:
                pdb.gimp_item_transform_rotate_simple(temp_layer3, (rotation % 4), True, 0, 0)
            selection3 = pdb.gimp_image_select_rectangle(new_image, 2, 0, new_image.height/2, (new_image.width)/2, new_image.height/2)
            pdb.gimp_drawable_edit_clear(original_layer)
            pdb.gimp_selection_none(new_image)
            rotation += 1	

			
            # and merge visible layers
            pdb.gimp_image_merge_visible_layers(new_image,1)
	
            #scale pattern to user size
            pdb.gimp_image_scale(new_image, size, size)
			
            pdb.gimp_image_resize_to_layers(new_image)
		
		
					
            # Set end undo point
            pdb.gimp_image_undo_group_end(new_image)
			
			
        if do_exemplar:

			
            # create second empty new image to user dimensions
            e_image = pdb.gimp_image_new(i_width * t_size, i_height * t_size, 0)
            # get tile layer and make a copy for reference
            e_layer = pdb.gimp_image_get_active_layer(new_image)
            new_name = pdb.gimp_item_get_name(e_layer)
            e_layer_copy = pdb.gimp_layer_new_from_drawable(e_layer, e_image)
            # add layer to new second image
            e_image.add_layer(e_layer_copy, -1)
            # Set start undo point
            pdb.gimp_image_undo_group_start(e_image)
            # scale layer to user dimensions
            pdb.gimp_layer_scale(e_layer_copy, t_size, t_size, False)
			
            iloop, loop = 0,0
            # create and position new layers
            for loop in range(int(i_height)):
                for i_loop in range(int(i_width)):
                    e_temp_layer = pdb.gimp_layer_new_from_drawable(e_layer_copy, e_image)
                    # add layer
                    e_image.add_layer(e_temp_layer, -1)
                    pdb.gimp_item_transform_translate(e_temp_layer, i_loop*(t_size),0+(loop*(t_size)))
            # remove original layer to avoid potential issue with transparent areas
            pdb.gimp_image_remove_layer(e_image, e_layer_copy)
            pdb.gimp_image_merge_visible_layers(e_image,1)
            name_layer = pdb.gimp_image_get_active_layer(e_image)
            pdb.gimp_item_set_name(name_layer, new_name)
            e_display = pdb.gimp_display_new(e_image)
			
            # Set end undo point
            pdb.gimp_image_undo_group_end(e_image)
			
    else:
        
            
        message_text = "Please ensure that there is at least one visible layer in the image and try again."
        old_handler = pdb.gimp_message_get_handler()	
        pdb.gimp_message_set_handler(MESSAGE_BOX)
        pdb.gimp_message(message_text)
        pdb.gimp_message_set_handler(old_handler)      
	
    # Set end undo point
    #pdb.gimp_image_undo_group_end(image)

    # Restore the GIMP's settings
    gimp.context_pop()
	
######################################################################################################################################

    #### Useful Functions
# -------------------------------------------------------------------------------------
	
def fn_selection_info(im):
    # returns selection info
    selection,x1,y1,x2,y2 = pdb.gimp_selection_bounds(im)
    s_width, s_height = x2 - x1, y2 - y1
	
    return selection, x1, x2, y1, y2, s_width, s_height


######################################################################################################################################
	
def fn_selection_amount(sel_value,r_ness,adjustment):
    # 
    value = (math.sqrt(sel_value)*r_ness)*adjustment
    return value

######################################################################################################################################

def proc_messagebox(message):
    old_handler = pdb.gimp_message_get_handler()
    pdb.gimp_message_set_handler(2)
    pdb.gimp_message(str(message))
    pdb.gimp_message_set_handler(old_handler)
	
######################################################################################################################################
		
def proc_trim_away(img, layer, amount):
    #### Trim off pixels from a layer
    # set our feather settings
    our_feather = True
    our_feather_radius = 2
    #get feather settings
    feather = pdb.gimp_context_get_feather()
    feather_x, feather_y = pdb.gimp_context_get_feather_radius()
    # make sure that feather is set
    if not feather: pdb.gimp_context_set_feather(our_feather)
    # use our feather radius
    pdb.gimp_context_set_feather_radius(our_feather_radius, our_feather_radius)
    # select the contents of the layer
    pdb.gimp_image_select_item(img, 2, layer)
    # shrink the selection
    pdb.gimp_selection_shrink(img, amount)
    # invert selection
    pdb.gimp_selection_invert(img)
    # trim away pixels
    successful = pdb.gimp_edit_cut(layer)
    # deselect
    pdb.gimp_selection_none(img)
    # reset feather settings
    pdb.gimp_context_set_feather(feather)
    pdb.gimp_context_set_feather_radius(feather_x, feather_y)
	
    #### End of Proc_trim_away
	
######################################################################################################################################


######################################################################################################################################
	
register(
    "python_fu_SeamStress_v1_3",
    "Provides options for making block tiles",
    "Provides options for making block tiles",
    "Skinnyhouse",
    "Open source (BSD 3-clause license)",
    "2021",
    "<Image>/Filters/SeamStress v1.3...",
    "RGB*",
    [
    (PF_OPTION, "option", "Tile type: ", 0, ("Diagonal", "Cut and swap", "Half brick", "Half drop", "Turnover (all)", "Turnover (L>R)", "Turnover (R>L)")),
    (PF_FLOAT, "angle", "Diagonal rotation: ", -45.0),
    (PF_OPTION, "option2", "Turnover rotation: ", 0, ("0 degrees", "90 degrees", "180 degrees", "270 degrees")),
    (PF_OPTION, "option3", "Half style flip: ",  0, ("No flip applied", "Horizontal flip", "Vertical flip")),
    (PF_SPINNER, "size", "Tile Size: ", 640, (50, 1000, 10)),
    (PF_TOGGLE, "example_option", "Create a second image filled with tiles: ", 0),
    (PF_SPINNER, "i_width", "Image width in tiles: ", 4, (2, 40, 1)),
    (PF_SPINNER, "i_height", "Image height in tiles: ", 2, (2, 40, 1)),
    (PF_SPINNER, "t_size", "Image tile size: ", 250, (50, 500, 10)),
    ],
    [],
    do_stuff)

main()
